@@ -1,4 +1,6 @@
 """Module that implements a basic command line interface (CLI) for AiiDAlab."""
+from __future__ import annotations
+
 import json
 import logging
 import shutil
@@ -66,7 +68,7 @@ def _list_apps(apps_path):
         )
 
 
-@click.group()
+@click.group(context_settings={"help_option_names": ["-h", "--help"]})
 @click.version_option(version=__version__, prog_name="AiiDAlab")
 @click.option("-v", "--verbose", count=True)
 def cli(verbose):
@@ -196,7 +198,7 @@ def _find_registered_app_from_id(name):
     """Find app for a given requirement."""
     try:
         app = AiidaLabApp.from_id(name)
-        if app.is_registered:
+        if app.is_registered():
             return app
         else:
             raise click.ClickException(
