@@ -1,5 +1,7 @@
 """Module to manage AiiDAlab apps."""
 
+from __future__ import annotations
+
 import errno
 import io
 import logging
@@ -8,6 +10,7 @@ import shutil
 import sys
 import tarfile
 import tempfile
+import threading
 from contextlib import contextmanager
 from dataclasses import asdict, dataclass, field
 from enum import Enum, Flag, auto
@@ -16,6 +19,7 @@ from pathlib import Path
 from subprocess import CalledProcessError
 from threading import Thread
 from time import sleep
+from typing import Any, Generator
 from urllib.parse import urldefrag, urlsplit, urlunsplit
 from uuid import uuid4
 
@@ -24,19 +28,16 @@ import traitlets
 from dulwich.errors import NotGitRepository
 from packaging.requirements import Requirement
 from packaging.version import parse
-from watchdog.events import FileSystemEventHandler
+from watchdog.events import EVENT_TYPE_OPENED, FileSystemEventHandler
 from watchdog.observers import Observer
 from watchdog.observers.polling import PollingObserver
 
-from .config import AIIDALAB_APPS
-from .environment import Environment
-from .git_util import GitManagedAppRepo as Repo
-from .git_util import git_clone
-from .metadata import Metadata
-from .utils import (
+from aiidalab.utils import (
     FIND_INSTALLED_PACKAGES_CACHE,
+    Package,
     PEP508CompliantUrl,
     find_installed_packages,
+    get_package_by_name,
     load_app_registry_entry,
     load_app_registry_index,
     run_pip_install,
@@ -48,8 +49,16 @@ from .utils import (
     throttled,
 )
 
+from .config import AIIDALAB_APPS
+from .environment import Environment
+from .git_util import GitManagedAppRepo as Repo
+from .git_util import git_clone
+from .metadata import Metadata
+
 logger = logging.getLogger(__name__)
 
+_CORE_PACKAGES = [Package("aiida-core"), Package("jupyter-client")]
+
 
 # A version is usually of type str, but it can also be a value
 # of this Enum to indicate special app states in which the
@@ -70,11 +79,10 @@ class AppRemoteUpdateStatus(Flag):
 
 @dataclass
 class _AiidaLabApp:
-
-    metadata: dict
+    metadata: dict[str, Any]
     name: str
     path: Path
-    releases: dict = field(default_factory=dict)
+    releases: dict[str, Any] = field(default_factory=dict)
 
     @classmethod
     def from_registry_entry(cls, path, registry_entry):
@@ -88,7 +96,7 @@ class _AiidaLabApp:
         )
 
     @classmethod
-    def _registry_entry_from_path(cls, path):
+    def _registry_entry_from_path(cls, path: Path) -> dict[str, Any]:
         try:
             return {
                 "name": path.stem,
@@ -117,7 +125,6 @@ class _AiidaLabApp:
 
         return cls.from_registry_entry(path=app_path, registry_entry=registry_entry)
 
-    @property
     def is_registered(self):
         try:
             app_registry_index = load_app_registry_index()
@@ -134,8 +141,8 @@ class _AiidaLabApp:
         except NotGitRepository:
             return None
 
-    def installed_version(self):
-        if self._repo and self.is_registered:
+    def installed_version(self) -> AppVersion | str:
+        if self._repo and self.is_registered():
             if self.dirty():
                 return AppVersion.UNKNOWN
             else:
@@ -154,19 +161,37 @@ class _AiidaLabApp:
                         f"Encountered error while determining version: {error}"
                     )
                     return AppVersion.UNKNOWN
-        elif self.path.exists():
+        elif self.is_installed():
             return self.metadata.get("version", AppVersion.UNKNOWN)
+
         return AppVersion.NOT_INSTALLED
 
-    def available_versions(self):
-        if self.is_registered:
-            yield from sorted(self.releases, key=parse, reverse=True)
+    def available_versions(
+        self,
+        python_bin: str | None = None,
+        prereleases: bool = False,
+    ) -> Generator[str, None, None]:
+        """Return a list of available versions excluding the ones with core dependency conflicts."""
+        if self.is_registered():
+            for version in sorted(self.releases, key=parse, reverse=True):
+                version_requirements = [
+                    Requirement(r)
+                    for r in (
+                        self.releases[version]
+                        .get("environment", {})
+                        .get("python_requirements", [])
+                    )
+                ]
+                if (
+                    prereleases or not parse(version).is_prerelease
+                ) and self._strict_dependencies_met(version_requirements, python_bin):
+                    yield version
 
     def dirty(self):
         if self._repo:
             return self._repo.dirty()
 
-    def is_installed(self):
+    def is_installed(self) -> bool:
         """The app is installed if the corresponding folder is present."""
         return self.path.exists()
 
@@ -182,10 +207,10 @@ class _AiidaLabApp:
         if self.is_installed():
 
             # Check whether app is registered.
-            if self.is_registered is None:
+            if self.is_registered() is None:
                 return AppRemoteUpdateStatus.CANNOT_REACH_REGISTRY
 
-            if self.is_registered is False:
+            if self.is_registered() is False:
                 return AppRemoteUpdateStatus.NOT_REGISTERED
 
             # Check whether the locally installed version is a registered release.
@@ -194,11 +219,7 @@ class _AiidaLabApp:
                 return AppRemoteUpdateStatus.DETACHED
 
             # Check whether the locally installed version is the latest release.
-            available_versions = [
-                version
-                for version in sorted(self.releases, key=parse, reverse=True)
-                if prereleases or not parse(version).is_prerelease
-            ]
+            available_versions = list(self.available_versions(prereleases=prereleases))
             if len(available_versions) and installed_version != available_versions[0]:
                 return AppRemoteUpdateStatus.UPDATE_AVAILABLE
 
@@ -230,22 +251,56 @@ class _AiidaLabApp:
         return matching_releases
 
     @staticmethod
-    def _find_incompatibilities_python(requirements, python_bin):
+    def _strict_dependencies_met(requirements: list[Requirement], python_bin) -> bool:
+        """Check whether the given requirements are compatible with the core dependencies of a package."""
+        from packaging.utils import canonicalize_name
+
+        packages = find_installed_packages(python_bin)
+        # Too avoid subtle bugs, we canonicalize the names of the requirements.
+        requirements_dict = {canonicalize_name(r.name): r for r in requirements}
+        for core in _CORE_PACKAGES:
+            installed_core_package = get_package_by_name(packages, core.canonical_name)
+            if (
+                core.canonical_name in requirements_dict
+                and installed_core_package is not None
+                and not installed_core_package.fulfills(
+                    requirements_dict[core.canonical_name]
+                )
+            ):
+                return False
+        return True
+
+    @staticmethod
+    def _find_incompatibilities_python(
+        requirements: list[str], python_bin: str
+    ) -> Generator[Requirement, None, None]:
         packages = find_installed_packages(python_bin)
         for requirement in map(Requirement, requirements):
-            f = [p for p in packages if p.fulfills(requirement)]
-            if not any(f):
+            pkg = get_package_by_name(packages, requirement.name)
+            if pkg is None:
+                yield requirement
+            elif not pkg.fulfills(requirement):
                 yield requirement
 
+    def is_detached(self):
+        """Check whether the app is detached from the registry."""
+        return self.remote_update_status() == AppRemoteUpdateStatus.DETACHED
+
     def find_incompatibilities(self, version, python_bin=None):
+        """Compatibility is checked by comparing the app requirements
+        with the packages installed in the python environment.
+
+        If the app is registered the list of requirements is fetched from the registry for the specific version.
+        If the app is not registered or if it is detached (i.e. locally modified),
+        the requirements list is read from the local repository (e.g. by parsing setup.cfg).
+        """
         if python_bin is None:
             python_bin = sys.executable
 
-        if not self.is_registered:
-            assert version == self.installed_version()
+        if not self.is_registered() or self.is_detached():
             environment = asdict(Environment.scan(self.path))
         else:
-            environment = self.releases[version].get("environment")
+            environment = self.releases[version].get("environment", {})
 
         for key, spec in environment.items():
             if key == "python_requirements":
@@ -259,29 +314,82 @@ class _AiidaLabApp:
     def is_compatible(self, version, python_bin=None):
         return not any(self.find_incompatibilities(version, python_bin))
 
-    def _install_dependencies(self, python_bin, stdout=None):
+    def find_dependencies_to_install(
+        self, version_to_install: str, python_bin: str | None = None
+    ) -> list[dict[str, Package | Requirement]]:
+        """Returns a list of dependencies that need to be installed.
+
+        If an unsupported version of a dependency is installed, it will look
+        something like: {installed=<Package...>, required=<Requirement(...)>}.
+
+        If the dependency is not present at all, it will look something like:
+        {installed=None, required=<Requirement(...)>}.
+        """
+        if python_bin is None:
+            python_bin = sys.executable
+
+        if not version_to_install:
+            return []
+
+        unmatched_dependencies = {
+            dep[1].name: dep[1]
+            for dep in self.find_incompatibilities(version_to_install, python_bin)
+        }
+        installed_packages = find_installed_packages(python_bin)
+        return [
+            {
+                "installed": get_package_by_name(installed_packages, name),
+                "required": requirement,
+            }
+            for name, requirement in unmatched_dependencies.items()
+        ]
+
+    def _install_dependencies(self, python_bin, stdout):
         """Try to install the app dependencies with pip (if specified)."""
 
+        def _should_run_reentry():
+            # We do not need to run reentry scan for aiida-core>=2"""
+            packages = find_installed_packages(python_bin=None)
+            installed_aiida = get_package_by_name(packages, "aiida-core")
+            # In practice this should never happen
+            if installed_aiida is None:
+                return False
+            aiida_v1 = Requirement("aiida-core<2.0.0b1")
+            # This is needed to handle pre-release versions such as 1.0.0b0
+            aiida_v1.specifier.prereleases = True
+            if installed_aiida.fulfills(aiida_v1):
+                return True
+            return False
+
         def _pip_install(*args, stdout):
             # The implementation of this function is taken and adapted from:
             # https://www.endpoint.com/blog/2015/01/getting-realtime-output-using-python/
 
             # Install package dependencies.
+            logger.info(f"Running 'pip install --user {' '.join(args)}'\n")
             process = run_pip_install(*args, python_bin=python_bin)
             for line in io.TextIOWrapper(process.stdout, encoding="utf-8"):
                 stdout.write(line)
+            process.wait()
             if process.returncode != 0:
                 raise RuntimeError("Failed to install dependencies.")
 
             # AiiDA plugins require reentry run to be found by AiiDA.
-            process = run_reentry_scan()
-            for line in io.TextIOWrapper(process.stdout, encoding="utf-8"):
-                stdout.write(line)
+            if _should_run_reentry():
+                logger.info("\nRunning 'reentry scan'\n")
+                process = run_reentry_scan()
+                for line in io.TextIOWrapper(process.stdout, encoding="utf-8"):
+                    stdout.write(line)
+                process.wait()
+                if process.returncode != 0:
+                    # Let's not fail the install because of this, just emit a warning.
+                    logger.warning("'reentry scan' failed")
 
             # Restarting the AiiDA daemon to import newly installed plugins.
             process = run_verdi_daemon_restart()
             for line in io.TextIOWrapper(process.stdout, encoding="utf-8"):
                 stdout.write(line)
+            process.wait()
             if process.returncode != 0:
                 raise RuntimeError("Failed to restart verdi daemon.")
 
@@ -316,8 +424,13 @@ class _AiidaLabApp:
             # detachment of child processes. For example, an app might trigger
             # a background process that is supposed to finish after the
             # installation is technically completed.
-            if run_post_install_script(post_install_file).returncode != 0:
-                raise CalledProcessError
+            process = run_post_install_script(post_install_file)
+            process.wait()
+            if process.returncode != 0:
+                logger.error(
+                    f'Post-install script "{post_install_file}" returned an error!'
+                )
+                raise CalledProcessError(process.returncode, str(post_install_file))
 
     def _install_from_path(self, path):
         if path.is_dir():
@@ -325,7 +438,28 @@ class _AiidaLabApp:
         else:
             with tempfile.TemporaryDirectory() as tmp_dir:
                 with tarfile.open(path) as tar_file:
-                    tar_file.extractall(path=tmp_dir)
+
+                    def is_within_directory(directory, target):
+
+                        abs_directory = os.path.abspath(directory)
+                        abs_target = os.path.abspath(target)
+
+                        prefix = os.path.commonprefix([abs_directory, abs_target])
+
+                        return prefix == abs_directory
+
+                    def safe_extract(
+                        tar, path=".", members=None, *, numeric_owner=False
+                    ):
+
+                        for member in tar.getmembers():
+                            member_path = os.path.join(path, member.name)
+                            if not is_within_directory(path, member_path):
+                                raise Exception("Attempted Path Traversal in Tar File")
+
+                        tar.extractall(path, members, numeric_owner=numeric_owner)
+
+                    safe_extract(tar_file, path=tmp_dir)
                     self._install_from_path(Path(tmp_dir))
 
     def _install_from_https(self, url):
@@ -395,29 +529,32 @@ class _AiidaLabApp:
 
             # Install dependencies
             if install_dependencies:
-                self._install_dependencies(python_bin, stdout=stdout)
+                self._install_dependencies(python_bin, stdout)
 
             # Run post-installation triggers.
             if post_install_triggers:
                 self._post_install_triggers()
 
-        except RuntimeError as error:
+        # NOTE: We want to catch everything, including keyboard interrupt,
+        # so we can rollback incomplete installation.
+        except BaseException as error:
             try:
                 if trash_path is None:
                     # App, was not previously installed, just remove it.
-                    logger.info("Removing partially installed app.")
+                    logger.warning("Removing partially installed app.")
                     self._move_to_trash()
                 else:
                     # Attempt rollback to previous version.
-                    logger.info("Performing rollback to previously installed version.")
+                    logger.warning(
+                        "Performing rollback to previously installed version."
+                    )
                     self._restore_from(trash_path)
-            except RuntimeError as error:
-                logger.warning(f"Rollback failed due to error: {error}!")
+            except RuntimeError as inner_error:
+                logger.error(f"Rollback failed due to error: {inner_error}!")
             finally:
                 raise RuntimeError(
                     f"Failed to install '{self.name}' (version={version}) at '{self.path}'"
-                    f", due to error: {error}"
-                )
+                ) from error
 
 
 class AppNotInstalledException(Exception):
@@ -431,20 +568,23 @@ class AiidaLabAppWatch:
     AiiDAlab app. This is achieved by monitoring the app repository
     for existance and changes.
 
+    If there is a change in the app repository, the app is refreshed.
+
     Arguments:
         app (AiidaLabApp):
             The AiidaLab app to monitor.
     """
 
-    class AppPathFileSystemEventHandler(FileSystemEventHandler):
+    class AppPathFileSystemEventHandler(FileSystemEventHandler):  # type: ignore
         """Internal event handeler for app path file system events."""
 
         def __init__(self, app):
             self.app = app
 
         def on_any_event(self, event):
-            """Refresh app for any event."""
-            self.app.refresh_async()
+            """Refresh app for any event except opened."""
+            if event.event_type != EVENT_TYPE_OPENED:
+                self.app.refresh_async()
 
     def __init__(self, app):
         self.app = app
@@ -452,6 +592,7 @@ class AiidaLabAppWatch:
         self._started = False
         self._monitor_thread = None
         self._observer = None
+        self._monitor_thread_stop = threading.Event()
 
     def __repr__(self):
         return f"<{type(self).__name__}(app={self.app!r})>"
@@ -462,7 +603,7 @@ class AiidaLabAppWatch:
         The ._observer thread is controlled by the ._monitor_thread.
         """
         assert os.path.isdir(self.app.path)
-        assert self._observer is None or not self._observer.isAlive()
+        assert self._observer is None or not self._observer.is_alive()
 
         event_handler = self.AppPathFileSystemEventHandler(self.app)
 
@@ -501,16 +642,17 @@ class AiidaLabAppWatch:
 
             def check_path_exists_changed():
                 is_dir = os.path.isdir(self.app.path)
-                while not self._monitor_thread.stop_flag:
+                while not self._monitor_thread_stop.is_set():
                     switched = is_dir != os.path.isdir(self.app.path)
                     if switched:
+                        # this is for when the app folder first time create or deleted
                         is_dir = not is_dir
                         self.app.refresh()
 
                     if is_dir:
-                        if self._observer is None or not self._observer.isAlive():
+                        if self._observer is None or not self._observer.is_alive():
                             self._start_observer()
-                    elif self._observer and self._observer.isAlive():
+                    elif self._observer and self._observer.is_alive():
                         self._stop_observer()
 
                     sleep(1)
@@ -520,7 +662,7 @@ class AiidaLabAppWatch:
                     self._observer.stop()
 
             self._monitor_thread = Thread(target=check_path_exists_changed)
-            self._monitor_thread.stop_flag = False
+            self._monitor_thread_stop.clear()
             self._monitor_thread.start()
 
         self._started = True
@@ -528,14 +670,14 @@ class AiidaLabAppWatch:
     def stop(self):
         """Stop watching the app repository for file system events."""
         if self._monitor_thread is not None:
-            self._monitor_thread.stop_flag = True
+            self._monitor_thread_stop.set()
 
     def is_alive(self):
         """Return True if this watch is still alive."""
         return self._monitor_thread and self._monitor_thread.is_alive()
 
     def join(self, timeout=None):
-        """Join the watch after stopping.
+        """Join the watch and observer after stopping.
 
         This function will timeout if a timeout argument is provided. Use the
         is_alive() function to determien whether the watch was stopped within
@@ -543,9 +685,11 @@ class AiidaLabAppWatch:
         """
         if self._monitor_thread is not None:
             self._monitor_thread.join(timeout=timeout)
+        if self._observer is not None:
+            self._observer.join(timeout=timeout)
 
 
-class AiidaLabApp(traitlets.HasTraits):
+class AiidaLabApp(traitlets.HasTraits):  # type: ignore
     """Manage installation status of an AiiDAlab app.
 
     Arguments:
@@ -560,22 +704,24 @@ class AiidaLabApp(traitlets.HasTraits):
             If true (default), automatically watch the repository for changes.
     """
 
-    path = traitlets.Unicode(allow_none=True, readonly=True)
+    path = traitlets.Unicode(allow_none=True).tag(readonly=True)
     install_info = traitlets.Unicode()
 
-    available_versions = traitlets.List(traitlets.Unicode)
+    available_versions = traitlets.List(traitlets.Unicode())
     installed_version = traitlets.Union(
         [traitlets.Unicode(), traitlets.UseEnum(AppVersion)]
-    )
+    )  # installed_version is updated from _AiiDALabApp only
+    version_to_install = traitlets.Unicode(allow_none=True)
+    dependencies_to_install = traitlets.List()
     remote_update_status = traitlets.UseEnum(
-        AppRemoteUpdateStatus, readonly=True, allow_none=True
-    )
+        AppRemoteUpdateStatus, allow_none=True
+    ).tag(readonly=True)
     has_prereleases = traitlets.Bool()
     include_prereleases = traitlets.Bool()
 
-    busy = traitlets.Bool(readonly=True)
-    detached = traitlets.Bool(readonly=True, allow_none=True)
-    compatible = traitlets.Bool(readonly=True, allow_none=True)
+    busy = traitlets.Bool().tag(readonly=True)
+    detached = traitlets.Bool(allow_none=True).tag(readonly=True)
+    compatible = traitlets.Bool(allow_none=True).tag(readonly=True)
     compatibility_info = traitlets.Dict()
 
     def __init__(self, name, app_data, aiidalab_apps_path, watch=True):
@@ -585,8 +731,19 @@ class AiidaLabApp(traitlets.HasTraits):
         super().__init__()
 
         self.name = self._app.name
-        self.logo = self._app.metadata["logo"]
-        self.categories = self._app.metadata["categories"]
+
+        self._busy_count = 0
+        self._busy_count_lock = threading.Lock()
+
+        try:
+            self.logo = self._app.metadata["logo"]
+        except KeyError:
+            raise ValueError(f"Did not find logo in {self.name} metadata")
+        try:
+            self.categories = self._app.metadata["categories"]
+        except KeyError:
+            raise ValueError(f"Did not find categories in {self.name} metadata")
+
         self.is_installed = self._app.is_installed
         self.path = str(self._app.path)
         self.refresh_async()
@@ -595,7 +752,7 @@ class AiidaLabApp(traitlets.HasTraits):
             self._watch = AiidaLabAppWatch(self)
             self._watch.start()
         else:
-            self._watch = None
+            self._watch = None  # type: ignore
 
     def __str__(self):
         return f"<AiidaLabApp name='{self._app.name}'>"
@@ -614,21 +771,50 @@ class AiidaLabApp(traitlets.HasTraits):
     def _default_detached(self):
         """Provide default value for detached traitlet."""
         if self.is_installed():
-            return self._app.dirty() or self._installed_version() is AppVersion.UNKNOWN
+            return (
+                self._app.dirty() or self._get_installed_version() is AppVersion.UNKNOWN
+            )
         return None
 
     @traitlets.default("busy")
     def _default_busy(self):  # pylint: disable=no-self-use
         return False
 
+    @traitlets.validate("version_to_install")
+    def _validate_version_to_install(self, proposal):
+        """Validate the version to install."""
+        with self._show_busy():
+            if proposal["value"] is None:
+                return
+
+            if proposal["value"] not in self.available_versions:
+                raise traitlets.TraitError(
+                    f"Version {proposal['value']} is not available for {self.name} app."
+                )
+            return proposal["value"]
+
+    @traitlets.observe("version_to_install")
+    def _observe_version_to_install(self, change):
+        if change["old"] != change["new"]:
+            self.refresh()
+
     @contextmanager
     def _show_busy(self):
         """Apply this decorator to indicate that the app is busy during execution."""
-        self.set_trait("busy", True)
+        # we need to use a lock here, because the busy trait is not thread-safe
+        # we may use _show_busy in different threads, e.g. when installing and auto-status refresh
+        with self._busy_count_lock:
+            self._busy_count += 1
+            self.set_trait("busy", True)
+
         try:
             yield
         finally:
-            self.set_trait("busy", False)
+            with self._busy_count_lock:
+                self._busy_count -= 1
+
+                if self._busy_count == 0:
+                    self.set_trait("busy", False)
 
     def in_category(self, category):
         # One should test what happens if the category won't be defined.
@@ -650,9 +836,9 @@ class AiidaLabApp(traitlets.HasTraits):
             )
             FIND_INSTALLED_PACKAGES_CACHE.clear()
             self.refresh()
-            return self._installed_version()
+            return self._get_installed_version()
 
-    def update_app(self, _=None, stdout=None):
+    def update_app(self, _=None, stdout=None) -> AppVersion | str:
         """Perform app update."""
         with self._show_busy():
             # Installing with version=None automatically selects latest
@@ -669,15 +855,15 @@ class AiidaLabApp(traitlets.HasTraits):
             self._app.uninstall()
             self.refresh()
 
-    def _installed_version(self):
+    def _get_installed_version(self) -> AppVersion | str:
         """Determine the currently installed version."""
         return self._app.installed_version()
 
-    @traitlets.default("compatible")
-    def _default_compatible(self):  # pylint: disable=no-self-use
+    @traitlets.default("compatible")  # type: ignore
+    def _default_compatible(self) -> None:  # pylint: disable=no-self-use
         return None
 
-    def _is_compatible(self, app_version):
+    def _is_compatible(self, app_version: str) -> bool:
         """Determine whether the specified version is compatible."""
         try:
             incompatibilities = dict(
@@ -692,45 +878,38 @@ class AiidaLabApp(traitlets.HasTraits):
 
             return not any(incompatibilities)
         except KeyError:
-            return None  # compatibility indetermined for given version
+            return False  # compatibility indetermined for given version
 
-    def _remote_update_status(self):
-        """Determine whether there are updates available.
-
-        For this the app must be installed in a known version and there must be
-        available (and compatible) versions.
-        """
-        installed_version = self._installed_version()
-        if installed_version not in (AppVersion.UNKNOWN, AppVersion.NOT_INSTALLED):
-            available_versions = list(self.available_versions)
-            if len(available_versions):
-                return self._installed_version() != available_versions[0]
-        return False
-
-    def _refresh_versions(self):
-        self.installed_version = self._app.installed_version()
+    def _refresh_versions(self) -> None:
+        self.installed_version = (
+            self._get_installed_version()
+        )  # only update at this refresh method
         self.include_prereleases = self.include_prereleases or (
             isinstance(self.installed_version, str)
             and parse(self.installed_version).is_prerelease
         )
 
-        all_available_versions = list(self._app.available_versions())
+        all_available_versions = list(self._app.available_versions(prereleases=True))
         self.has_prereleases = any(
             parse(version).is_prerelease for version in all_available_versions
         )
-        if self._app.is_registered:
-            self.available_versions = [
-                version
-                for version in all_available_versions
-                if self.include_prereleases or not parse(version).is_prerelease
-            ]
 
-    @throttled(calls_per_second=1)
-    def refresh(self):
+        self.available_versions = list(
+            self._app.available_versions(prereleases=self.include_prereleases)
+        )
+
+    def _refresh_dependencies_to_install(self) -> None:
+        self.dependencies_to_install = self._app.find_dependencies_to_install(
+            self.version_to_install
+        )
+
+    @throttled(calls_per_second=1)  # type: ignore
+    def refresh(self) -> None:
         """Refresh app state."""
         with self._show_busy():
             with self.hold_trait_notifications():
                 self._refresh_versions()
+                self._refresh_dependencies_to_install()
                 self.set_trait(
                     "compatible", self._is_compatible(self.installed_version)
                 )
@@ -743,11 +922,11 @@ class AiidaLabApp(traitlets.HasTraits):
                 self.set_trait(
                     "detached",
                     (self.installed_version is AppVersion.UNKNOWN)
-                    if (self._has_git_repo() and self._app.is_registered)
+                    if (self._has_git_repo() and self._app.is_registered())
                     else None,
                 )
 
-    def refresh_async(self):
+    def refresh_async(self) -> None:
         """Asynchronized (non-blocking) refresh of the app state."""
         refresh_thread = Thread(target=self.refresh)
         refresh_thread.start()
@@ -757,7 +936,7 @@ class AiidaLabApp(traitlets.HasTraits):
         """Return metadata dictionary. Give the priority to the local copy (better for the developers)."""
         return self._app.metadata
 
-    def _get_from_metadata(self, what):
+    def _get_from_metadata(self, what: str) -> str:
         """Get information from metadata."""
         try:
             return f"{self._app.metadata[what]}"
@@ -765,28 +944,28 @@ class AiidaLabApp(traitlets.HasTraits):
             return f'Field "{what}" is not present in app metadata.'
 
     @property
-    def authors(self):
+    def authors(self) -> str:
         return self._get_from_metadata("authors")
 
     @property
-    def description(self):
+    def description(self) -> str:
         return self._get_from_metadata("description")
 
     @property
-    def title(self):
+    def title(self) -> str:
         return self._get_from_metadata("title")
 
     @property
-    def url(self):
+    def url(self) -> str:
         """Provide explicit link to Git repository."""
         return self._get_from_metadata("external_url")
 
     @property
-    def more(self):
+    def more(self) -> str:
         return f"""<a href=./single_app.ipynb?app={self.name}>Manage App</a>"""
 
     @property
-    def _repo(self):
+    def _repo(self) -> Repo:
         """Returns Git repository."""
         if not self.is_installed():
             raise AppNotInstalledException("The app is not installed")
