@@ -1,7 +1,8 @@
 """App metadata specification"""
+from __future__ import annotations
+
 from configparser import ConfigParser
 from dataclasses import dataclass, field
-from typing import List
 
 __all__ = [
     "Metadata",
@@ -64,7 +65,12 @@ def _parse_setup_cfg(setup_cfg):
         "state", _map_development_state(metadata_pep426.get("classifiers", []))
     )
 
-    yield "categories", aiidalab.get("categories", [])
+    # Allow passing single category and convert to list
+    # and allow parse line separated string as list
+    categories = aiidalab.get("categories", [])
+    if isinstance(categories, str):
+        categories = [c for c in categories.split("\n") if c]
+    yield "categories", categories
 
 
 @dataclass
@@ -73,13 +79,13 @@ class Metadata:
 
     title: str
     description: str
-    authors: str = None
-    state: str = None
-    documentation_url: str = None
-    external_url: str = None
-    logo: str = None
-    categories: List[str] = field(default_factory=list)
-    version: str = None
+    authors: None | str = None
+    state: None | str = None
+    documentation_url: None | str = None
+    external_url: None | str = None
+    logo: None | str = None
+    categories: list[str] = field(default_factory=list)
+    version: None | str = None
 
     _search_dirs = (".aiidalab", "./")
 
