@@ -1,4 +1,5 @@
 """Helpful utilities for the AiiDAlab tools."""
+from __future__ import annotations
 
 import json
 import logging
@@ -11,11 +12,13 @@ from functools import wraps
 from pathlib import Path
 from subprocess import run
 from threading import Lock
+from typing import Any
 from urllib.parse import urlsplit, urlunsplit
 
 import requests
 from cachetools import TTLCache, cached
-from packaging.utils import canonicalize_name
+from packaging.requirements import Requirement
+from packaging.utils import NormalizedName, canonicalize_name
 
 from .config import AIIDALAB_REGISTRY
 from .environment import Environment
@@ -23,7 +26,7 @@ from .fetch import fetch_from_url
 from .metadata import Metadata
 
 logger = logging.getLogger(__name__)
-FIND_INSTALLED_PACKAGES_CACHE = TTLCache(maxsize=32, ttl=60)
+FIND_INSTALLED_PACKAGES_CACHE = TTLCache(maxsize=32, ttl=60)  # type: ignore
 
 # Warning: try-except is a fix for Quantum Mobile release v19.03.0 where
 # requests_cache is not installed.
@@ -52,7 +55,7 @@ else:
     )
 
 
-def load_app_registry_index():
+def load_app_registry_index():  # type: ignore
     """Load apps' information from the AiiDAlab registry."""
     try:
         return requests.get(f"{AIIDALAB_REGISTRY}/apps_index.json").json()
@@ -60,7 +63,7 @@ def load_app_registry_index():
         raise RuntimeError(f"Unable to load registry index: '{error}'")
 
 
-def load_app_registry_entry(app_id):
+def load_app_registry_entry(app_id: str):  # type: ignore
     """Load registry enty for app with app_id."""
     try:
         return requests.get(f"{AIIDALAB_REGISTRY}/apps/{app_id}.json").json()
@@ -75,7 +78,9 @@ class PEP508CompliantUrl(str):
     pass
 
 
-def parse_app_repo(url, metadata_fallback=None):
+def parse_app_repo(
+    url: str, metadata_fallback: dict[str, Any] | None = None
+) -> dict[str, Any] | None:
     """Parse an app repo for metadata and other information.
 
     Use this function to parse a local or remote app repository for the app
@@ -96,7 +101,7 @@ def parse_app_repo(url, metadata_fallback=None):
             metadata = asdict(Metadata.parse(repo))
         except TypeError as error:
             logger.debug(f"Failed to parse metadata for '{url}': {error}")
-            metadata = metadata_fallback
+            metadata = metadata_fallback  # type: ignore
 
         return {
             "metadata": metadata,
@@ -115,16 +120,16 @@ class throttled:  # pylint: disable=invalid-name
 
     """
 
-    def __init__(self, calls_per_second=1):
+    def __init__(self, calls_per_second: int = 1):
         self.calls_per_second = calls_per_second
-        self.last_start = defaultdict(lambda: -1)
-        self.locks = defaultdict(Lock)
+        self.last_start = defaultdict(lambda: -1)  # type: ignore
+        self.locks = defaultdict(Lock)  # type: ignore
 
-    def __call__(self, func):
+    def __call__(self, func):  # type: ignore
         """Return decorator function."""
 
         @wraps(func)
-        def wrapped(instance, *args, **kwargs):
+        def wrapped(instance, *args, **kwargs):  # type: ignore
             if self.last_start[hash(instance)] >= 0:
                 elapsed = time.perf_counter() - self.last_start[hash(instance)]
                 to_wait = 1.0 / self.calls_per_second - elapsed
@@ -147,23 +152,43 @@ class throttled:  # pylint: disable=invalid-name
 class Package:
     """Helper class to check whether a given package fulfills a requirement."""
 
-    def __init__(self, name, version):
-        self.name = name
+    def __init__(self, name: str, version: str | None = None):
+        """If version is None, means not confinement for the version therefore
+        the package always fulfill."""
+        self._name = name  # underscore to avoid name clash with property canonical_name
         self.version = version
 
-    def __str__(self):
-        return f"{type(self).__name__}({self.name}, {self.version})"
+    def __repr__(self) -> str:
+        return f"{type(self).__name__}({self.canonical_name}, {self.version})"
 
-    def fulfills(self, requirement):
-        """Returns True if this entry fullfills the requirement."""
-        return (
-            canonicalize_name(self.name) == canonicalize_name(requirement.name)
-            and self.version in requirement.specifier
+    def __str__(self) -> str:
+        return f"{self.canonical_name}=={self.version}"
+
+    @property
+    def canonical_name(self) -> NormalizedName:
+        """Return the cananicalized name of the package."""
+        return canonicalize_name(self._name)
+
+    def fulfills(self, requirement: Requirement) -> bool:
+        """Returns True if this entry fulfills the requirement."""
+        return self.canonical_name == canonicalize_name(requirement.name) and (
+            self.version in requirement.specifier or self.version is None
         )
 
 
 @cached(cache=FIND_INSTALLED_PACKAGES_CACHE)
-def find_installed_packages(python_bin=None):
+def find_installed_packages(python_bin: str | None = None) -> dict[str, Package]:
+    """Return all currently installed packages."""
+    output = _pip_list(python_bin)
+    return {
+        canonicalize_name(package["name"]): Package(
+            name=canonicalize_name(package["name"]), version=package["version"]
+        )
+        for package in output
+    }
+
+
+def _pip_list(python_bin: str | None = None) -> Any:
     """Return all currently installed packages."""
     if python_bin is None:
         python_bin = sys.executable
@@ -172,13 +197,25 @@ def find_installed_packages(python_bin=None):
         encoding="utf-8",
         capture_output=True,
     ).stdout
-    return [
-        Package(name=package["name"], version=package["version"])
-        for package in json.loads(output)
-    ]
+
+    return json.loads(output)
 
 
-def split_git_url(git_url):
+def get_package_by_name(packages: dict[str, Package], name: str) -> Package | None:
+    """Return the package with the given name from the list of packages.
+    The name can be the canonicalized name or the requirement name which may not canonicalized.
+    We try to convert the name to the canonicalized in both side and compare them.
+
+    For example, the requirement name is 'jupyter-client' and the package name is 'jupyter_client'.
+    The implementation of this method is inspired by https://github.com/pypa/pip/pull/8054
+    """
+    for package in packages.values():
+        if package.canonical_name == canonicalize_name(name):
+            return package
+    return None
+
+
+def split_git_url(git_url):  # type: ignore
     """Split the base url and the ref pointer of a git url.
 
     For example: git+https://example.com/app.git@v1 is split into and returned
@@ -193,29 +230,28 @@ def split_git_url(git_url):
     return urlunsplit(parsed_url._replace(path=path)), ref
 
 
-def this_or_only_subdir(path):
+def this_or_only_subdir(path: Path) -> Path:
     members = list(path.iterdir())
     return members[0] if len(members) == 1 and members[0].is_dir() else path
 
 
-def run_pip_install(*args, python_bin=sys.executable):
+def run_pip_install(*args, python_bin=sys.executable):  # type: ignore
     return subprocess.Popen(
-        [python_bin, "-m", "pip", "install", *args],
+        [python_bin, "-m", "pip", "install", "--user", *args],
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
     )
 
 
-def run_reentry_scan():
+def run_reentry_scan():  # type: ignore
     return subprocess.Popen(
         ["reentry", "scan"],
-        bufsize=1,
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
     )
 
 
-def run_verdi_daemon_restart():
+def run_verdi_daemon_restart():  # type: ignore
     return subprocess.Popen(
         ["verdi", "daemon", "restart"],
         stdout=subprocess.PIPE,
@@ -223,7 +259,7 @@ def run_verdi_daemon_restart():
     )
 
 
-def run_post_install_script(post_install_script_path):
+def run_post_install_script(post_install_script_path):  # type: ignore
     return subprocess.Popen(
         f"./{post_install_script_path.resolve().stem}",
         cwd=post_install_script_path.resolve().parent,
